<?php
/**
 * @version    $Id$
 * @package    JSN_EasySlider
 * @author     JoomlaShine Team <support@joomlashine.com>
 * @copyright  Copyright (C) 2012 JoomlaShine.com. All Rights Reserved.
 * @license    GNU/GPL v2 or later http://www.gnu.org/licenses/gpl-2.0.html
 *
 * Websites: http://www.joomlashine.com
 * Technical Support:  Feedback - http://www.joomlashine.com/contact-us/get-support.html
 */

defined('_JEXEC') or die('Restricted access');

/**
 * slider model.
 *
 * @package  JSN_EasySlider
 * @since    1.0.0
 */
include_once JPATH_COMPONENT_ADMINISTRATOR . '/classes/jsn.easyslider.sliders.php';

class JSNEasySliderModelSlider extends JModelAdmin
{

    protected $option = "com_easyslider";

    /**
     * Method to get a table object, load it if necessary.
     *
     * @param   string $type The table name. Optional.
     * @param   string $prefix The class prefix. Optional.
     * @param   array $config Configuration array for model. Optional.
     *
     * @return  JTable  A JTable object
     */
    public function getTable($type = 'slider', $prefix = 'JSNEasySliderTable', $config = array())
    {
        return JTable::getInstance($type, $prefix, $config);
    }

    /**
     * Method to get the record form.
     *
     * @param   array $data Data for the form.
     * @param   boolean $loadData True if the form is to load its own data (default case), false if not.
     *
     * @return    mixed    A JForm object on success, false on failure
     */
    public function getForm($data = array(), $loadData = true)
    {
        $form = $this->loadForm('com_easyslider.slider', 'slider', array('control' => 'jform', 'load_data' => $loadData));

        return $form;
    }

    /**
     * Load pre-defined data to fill into form.
     *
     * @return  object
     */
    protected function loadFormData()
    {
        // Check the session for previously entered form data.
        $data = JFactory::getApplication()->getUserState('com_easyslider.edit.form.data', array());

        if (empty($data))
        {
            $data = $this->getItem();
        }
        return $data;
    }


    public function updateSliderData($data = array())
    {
        $result = array();
        try
        {
            if (!empty($data))
            {
                $query = $this->_db->getQuery(true);

                $fields = array(
                    $this->_db->quoteName('slider_title') . ' = ' . $this->_db->quote($data['slider_title']),
                    $this->_db->quoteName('slider_data') . ' = ' . $this->_db->quote($data['slider_data']),
                );

                // Conditions for which records should be updated.
                $conditions = array(
                    $this->_db->quoteName('slider_id') . ' = ' . $data['slider_id'],
                );

                $query->update($this->_db->quoteName('#__jsn_easyslider_sliders'))->set($fields)->where($conditions);

                $this->_db->setQuery($query);
                $this->_db->execute();
                $result['message'] = JText::_('JSN_EASYSLIDER_UPDATE_SUCCESSFULLY', true); //Update data successful
                $result['error'] = false;
            }
            else
            {
                $result['message'] = JText::_('JSN_EASYSLIDER_UPDATE_UNSUCCESSFULLY', true);
                $result['error'] = true;
            }

            return json_encode($result);
        } catch (Exception $e)
        {
            // catch any database errors.
            $result['message'] = JText::_('JSN_EASYSLIDER_UPDATE_UNSUCCESSFULLY', true);
            $result['error'] = true;
            return json_encode($result);
        }
    }

    public function createNewSlider($data = array())
    {
        $result = array();

        $objJSNEasySliderSliders = new JSNEasySliderSliders();
        $totalSliders = $objJSNEasySliderSliders->countSilderItems();

        /*Check if it is FREE edition then show warning message to alert that FREE edition only allows create maximum of 3 sliders*/
        $edition = defined('JSN_EASYSLIDER_EDITION') ? JSN_EASYSLIDER_EDITION : "free";
        if (strtolower($edition) == 'free')
        {
            if ($totalSliders !== false && $totalSliders >= 3)
            {
                $result['message'] = JText::_('JSN_EASYSLIDER_YOU_HAVE_REACHED_THE_LIMITATION_OF_3_SLIDERS_IN_FREE_EDITION', true);
                $result['error'] = true;
                return json_encode($result);
            }
        }

        try
        {
            $query = $this->_db->getQuery(true);

            // Insert columns.
            $columns = array('slider_title', 'published', 'access', 'slider_data');

            // Insert values.
            $values = array($this->_db->quote($data['slider_title']), $data['published'], $data['access'], $this->_db->quote($data['slider_data']));

            // Prepare the insert query.
            $query
                ->insert($this->_db->quoteName('#__jsn_easyslider_sliders'))
                ->columns($this->_db->quoteName($columns))
                ->values(implode(',', $values));

            $this->_db->setQuery($query);
            $this->_db->execute();
            $sliderID = $this->_db->insertid();

            $result['message'] = JText::_('JSN_EASYSLIDER_INSERT_SUCCESSFULLY', true); //Update data successful
            $result['error'] = false;
            $result['slider_id'] = $sliderID;
            return json_encode($result);
        } 
        catch (Exception $e)
        {
            // catch any database errors.
            $result['message'] = JText::_('JSN_EASYSLIDER_INSERT_UNSUCCESSFULLY', true);
            $result['error'] = true;
            return json_encode($result);
        }
    }
}
