void function ( exports, $, _, Backbone, undefined ) {

	var youtubeReady = false;

	//const log = console.log.bind( console );
	const log = _.noop;

	const TRANSITIONS = JSNES_TRANSITIONS_LIST;
	const TRANSITION_DEFAULTS = {
		type: 1, /* 1: basic; 2: complex; 3: cube; */

		// Basic transition params
		effect: 'fade-in-front',
		delay: 5000,
		duration: 1000,
		timing: 'ease',

		// Complex transition params
		rows: 1,
		cols: 1,
		delayRandom: false,
		delayY: 50,
		delayX: 50,

		// Cuboid transition params
		cubeDepth: 'auto',
		cubeAnimation: 'scale-rotate',
		cubeFace: 'left',
		cubeAxis: 'y',
		cubeRotate: 1
	};
	const ANIMATION_DEFAULTS = {
		effect: 'none',
		timing: 'ease',
		fade: true
	}

	var JSNEasySlider = Backbone.View.extend( {
		events: {
			'click .next': 'next',
			'click .prev': 'prev',
			'click .goto': function (e) {
				if (this.$el.hasClass('transition-active')) {
					return;
				}

				var index = parseInt($(e.currentTarget).attr('target'));
				this.goto(index);
			},
			'click a.item-content[href]': function( e ) {
				var href = $(e.currentTarget).attr('href');
				var match = href.match(/(@)(.*)/);
				if (match && match[1] == '@') {
					e.preventDefault();
					var index = parseInt(match[2]);
					if (index) {
						this.goto(index-1);
					}
				}
			}
		},
		constructor: function JSNEasySlider() {
			this.lazyRender = _.throttle( this.render, 100 );
			this.lazyReveal = _.throttle( this.reveal, 100 );
			Backbone.View.apply( this, arguments );
		},
		initialize: function () {
			log( 'Slider :: initialize' );

			this.slides = [];
			this.transitionView = new JSNEasySlider_TransitionView;

			this.fullHeight = this.$el.data( 'full-height' );
			this.fullWidth = this.$el.data( 'full-width' );

			this.width = this.$el.data( 'width' );
			this.height = this.$el.data( 'height' );

			this.containerWidth = this.$el.data( 'container-width' );
			this.containerHeight = this.$el.data( 'container-height' );

			this.tabletMode = this.$el.data( 'tablet-mode' ) || 'false';
			this.tabletUnder = parseInt( this.$el.data( 'tablet-under' ) );
			this.tabletWidth = this.$el.data( 'tablet-width' );
			this.tabletHeight = this.$el.data( 'tablet-height' );

			this.mobileMode = this.$el.data( 'mobile-mode' ) || 'false';
			this.mobileUnder = parseInt( this.$el.data( 'mobile-under' ) );
			this.mobileWidth = this.$el.data( 'mobile-width' );
			this.mobileHeight = this.$el.data( 'mobile-height' );

			this.maxWidth = this.$el.data( 'max-width' );
			this.maxHeight = this.$el.data( 'max-height' );

			this.minWidth = this.$el.data( 'min-width' );
			this.minHeight = this.$el.data( 'min-height' );

			this.startAt = parseInt( this.$el.data( 'start-at' ) );
			!_.isNaN(this.startAt) || (this.startAt = 1);
			this.startAt -= 1;
			log(this.startAt)

			this.transitionView.$el.appendTo( this.el );

			this.$( '.jsn-es-slide' ).each( _.bind( function ( i, el ) {
				this.slides.push( new JSNEasySlider_SlideView( { el: el, slider: this } ) );
				this.$('.buttons').append('<button class="goto" target="' + i + '">' + (i+1) + '</button>');
			}, this ) );

			$( window ).resize( _.bind( function() {
				this.render();
			}, this ) );
			_.delay(_.bind(this.start, this), 100);
		},
		scale: function() {
			_( this.slides ).invoke( 'scale' );
			return this;
		},
		resize: function () {
			log( 'Slider :: resize' );
			!this.fullHeight || (this.height = window.innerHeight);
			_( this.slides ).invoke( 'resize' );
			this.lazyReveal();
			return this;
		},
		reveal: function () {
			log( 'Slider :: reveal' );
			var top = this.$el.offset().top;
			if ( this.fullHeight && top > window.scrollY && top < window.scrollY + window.innerHeight / 2 ) {
				window.scrollTo( 0, parseInt( top ) );
			}
		},
		render: function () {
			log( 'Slider :: render' );
			this.$el
				.width( this.width )
				.height( this.getHeight() )
				.css( {
					minWidth: this.minWidth,
					minHeight: this.minHeight,
					maxWidth: this.maxWidth,
					maxHeight: this.maxHeight,
				} );
			if ( this.fullWidth ) {

				this.$el
					.css( {
						width: '100%',
						marginLeft: '0',
						marginRight: '0',
					} );

				var offsetLeft = this.$el.offset().left;
				var offsetRight = window.innerWidth - (this.$el.offset().left + this.$el.parent().width());

				this.$el
					.css( {
						width: window.innerWidth + 'px',
						marginLeft: -offsetLeft + 'px',
						marginRight: -offsetRight + 'px',
					} );
			}

			this.resize();
			this.scale();

			_( this.slides ).invoke( 'render' );

			return this;
		},

		start: function () {
			this.render();
			this.goto( this.startAt);
		},
		prev: function () {
			if ( this.$el.hasClass( 'transition-active' ) )
				return;
			this.goto( this.current > 0 ? this.current - 1 : this.slides.length - 1 );
			return this;
		},
		next: function () {
			if ( this.$el.hasClass( 'transition-active' ) )
				return;
			if ( this.current + 1 == this.total && !this.loop )
				return;
			this.goto( this.current < this.slides.length - 1 ? this.current + 1 : 0 );
			return this;
		},
		goto: function ( index ) {
			if ( this.$el.is( '.jsn-es-animation-active' ) )
				return;
			if ( index == this.current || index >= this.slides.length )
				return;
			var toSlide = this.slides[ index ];
			var fromSlide = this.getActiveSlide();
			this.current = index;
			this.$el.addClass('loading');
			toSlide.load(_.bind(function() {

				this.$el.removeClass('loading');
				toSlide.start( fromSlide );

				this.$('.buttons-container button:eq(' + index + ')')
					.addClass('active')
					.siblings()
					.removeClass('active');
			}, this));

			return this;
		},

		getHeight: function () {
			var width = window.innerWidth;

			if ( this.fullHeight )
				return window.innerHeight;

			else if ( this.height )
				return this.height;

			else {

				if ( this.mobileMode == true && width < this.mobileUnder)
					return this.mobileHeight || this.containerHeight;

				if ( this.tabletMode == true && width < this.tabletUnder)
					return this.tabletHeight || this.containerHeight;

				if ( this.containerHeight )
					return this.containerHeight;
			}
		},
		getActiveSlide: function () {
			return this.slides[ this.current ];
		},
	} );

	var JSNEasySlider_SlideView = Backbone.View.extend( {
		events: {},
		constructor: function JSNEasySlider_SlideView() {
			//this.scale = _.debounce(this.scale);
			Backbone.View.apply( this, arguments );
		},
		initialize: function ( options ) {

			log( 'Slide :: initialize' );

			this.slider = options.slider;
			this.items = [];
			this.timer = [];
			this.responsive = 'default';
			this.scaleFactor = 1;

			this.$container = this.$( '.jsn-es-container' )
			this.$image = this.$el.children( 'img' ).remove();
			this.$video = this.$el.children( 'video' );

			this.backgroundColor = this.$el.data( 'bg-color' ) || 'transparent';
			this.backgroundSize = this.$el.data( 'bg-size' ) || 'cover';
			this.backgroundPosition = this.$el.data( 'bg-position' ) || 'center';
			this.backgroundImage = this.$image.attr( 'src' ) || this.$image.data( 'src' );

			this.containerWidth = this.slider.containerWidth;
			this.containerHeight = this.slider.containerHeight;

			this.transition = this.$el.data( 'transition' ) || {};

			_.defaults( this.transition, TRANSITION_DEFAULTS );

			this.transition.durationTotal = this.transition.duration;
			if (this.transition.type != 1) {
				var delayX = Math.abs(this.transition.delayX * ((this.transition.cols || 1) - 1));
				var delayY = Math.abs(this.transition.delayY * ((this.transition.rows || 1) - 1));
				this.transition.durationTotal = (delayX + delayY + this.transition.duration);
			}

			this.$( '.jsn-es-item' ).each( _.bind( function ( i, el ) {
				this.items.push( new JSNEasySlider_ItemVIew( { el: el, slide: this } ) );
			}, this ) );

			//$(window).resize( _.debounce(_.bind(this.pauseItems, this), 500, true) );
			//$(window).resize( _.debounce(_.bind(this.resumeItems, this), 500) );

			this.render();
			this.$el.addClass( 'hidden' );
		},
		scale: function() {

			var outerWidth = this.slider.$el.width();
			var canvasWidth = this.$container.width();

			this.scaleFactor = outerWidth / canvasWidth
			this.scaleFactor <= 1 || (this.scaleFactor = 1);

			this.$container.css({
				transform: 'translate(-50%,-50%) scale('+ this.scaleFactor +')',
				transformOrigin: 'center'
			});

			// Set the slider height to its default height
			// Then get its height in pixel multiply by the scale factor
			this.slider.$el.height( this.slider.getHeight() );
			this.slider.$el.height( this.slider.$el.height() * this.scaleFactor );

			return this;
		},
		resize: function () {

			log( 'Slide :: resize' );

			var sliderWidth = window.innerWidth;

			if ( this.slider.mobileMode == true && sliderWidth < this.slider.mobileUnder ) {
				this.containerWidth = this.slider.mobileWidth;
				this.containerHeight = this.slider.mobileHeight || this.slider.containerHeight;
				this.responsive = 'mobile';
			}

			else if ( this.slider.tabletMode == true && sliderWidth < this.slider.tabletUnder ) {
				this.containerWidth = this.slider.tabletWidth;
				this.containerHeight = this.slider.tabletHeight || this.slider.containerHeight;
				this.responsive = 'tablet';
			}
			else {
				this.containerWidth = this.slider.containerWidth;
				this.containerHeight = this.slider.containerHeight;
				this.responsive = 'default';
			}

			return this.render();
		},
		render: function () {

			log( 'Slide :: render' );

			this.$el.get( 0 ).style.cssText = null;

			this.$el.css( {
				backgroundColor: this.backgroundColor,
				backgroundSize: this.backgroundSize,
				backgroundPosition: this.backgroundPosition,
				backgroundImage: this.backgroundImage ? 'url(' + this.backgroundImage + ')' : '',
			} );

			this.$container
				.width( this.containerWidth )
				.height( this.containerHeight );

			if ( this.$video.length ) {
				var videoRatio = this.$video.width() / this.$video.height();
				var sliderRatio = this.slider.$el.width() / this.slider.$el.height();
				if ( videoRatio > sliderRatio )
					this.$video.css( {
						height: '100%',
						width: 'auto'
					} );
				else
					this.$video.css( {
						width: '100%',
						height: 'auto'
					} );
			}

			_( this.items ).invoke( 'render' );
			return this;
		},
		load: function( callback ) {
			var imageCount = 1;
			var callback = _.once(callback);

			this.$el.find('img[data-src]').add(this.$image).each(function(){
				var dataSrc = this.getAttribute('data-src');
				if (this.src || !dataSrc)
					return;
				this.removeAttribute('data-src');
				this.onload = function() {
					gotoNextSlide();
				};
				this.onerror = function() {
					gotoNextSlide();
				}
				this.src =  dataSrc;
				imageCount++;
			});
			var gotoNextSlide = _.after(imageCount, _.bind(callback, this));
			gotoNextSlide();
			return this;
		},
		start: function ( fromSlide ) {
			var toSlide = this;
			var transitionView = this.slider.transitionView;
			var transition, duration = 0;

			toSlide.$el
				.removeClass( 'hidden' )
				.addClass( 'active' );
			toSlide.render();

			if ( fromSlide ) {
				transition = fromSlide.transition;
				duration = transition.durationTotal;
			}

			_( this.items ).invoke( 'start', duration, this.transition.delay );

			if ( fromSlide ) {

				fromSlide.end();
				fromSlide.$el
					.removeClass( 'hidden' )
					.removeClass( 'active' );
				fromSlide.render();

				transitionView.play( fromSlide.el, toSlide.el, transition );

				this.slider.$el
					.addClass( 'jsn-es-animation-active' );

				this.delay( function () {
					this.slider.$el
						.removeClass( 'jsn-es-animation-active' );

					transitionView.$el.addClass( 'hidden' );
					fromSlide.$el.addClass( 'hidden' );
					toSlide.$el.removeClass( 'hidden' );
					toSlide.render();

				}, duration );
			}
			this.timerEnd = this.delay( function () {
				toSlide.slider.next();
			}, this.transition.delay + duration );
			return this;
		},
		end: function () {
			_( this.items ).invoke( 'end' );
			_( this.timer ).invoke( 'stop' );
			while ( this.timer.length )
				this.timer.pop();
			delete this.timerEnd;
			return this;
		},
		pause: function () {
			this.timerEnd && this.timerEnd.pause();
			return this;
		},
		pauseItems: function () {
			this.timerEnd && this.timerEnd.pause();
			_( this.items ).invoke( 'pause' );
			this.$( '.item-animation' ).css( 'animation-play-state', 'paused' );
			return this;
		},
		resumeItems: function () {
			this.timerEnd && this.timerEnd.start();
			_( this.items ).invoke( 'resume' );
			this.$( '.item-animation' ).css( 'animation-play-state', 'running' );
			return this;
		},

		delay: function ( cb, delay, context ) {
			this.timer || (this.timer = []);
			var timer = new Timer( {
				onend: _.bind( function() {
					//this.timer.splice( this.timer.indexOf(timer), 1 );
					_.isFunction(cb) && cb.call( context || this );
				}, this )
			} );
			this.timer.push( timer.start( delay / 1000 ) );
			return timer;
		},
	} );

	var JSNEasySlider_ItemVIew = Backbone.View.extend( {
		events: {},
		constructor: function JSNEasySlider_ItemVIew() {
			Backbone.View.apply( this, arguments );
		},
		initialize: function ( options ) {

			log( 'Item :: initialize' );

			this.timer = [];
			this.slide = options.slide;
			this.type = this.$el.data( 'type' ) || 'html';
			this.index = this.$el.data( 'index' ) || '1';
			this.className = this.$el.attr( 'class' );

			this.origin = (this.$el.data( 'origin' ) || '0,0').split( ',' ).map( function ( value ) {
				return parseFloat( value )
			} );

			this.style = this.$el.data( 'style' ) || {};
			this.styleT = this.$el.data( 'style-t' ) || {};
			this.styleM = this.$el.data( 'style-m' ) || {};

			this.customClass = this.$el.data('classname');

			this.build = {
				in: this.$el.data( 'animation-in' ) || {},
				out: this.$el.data( 'animation-out' ) || {}
			}
			_.defaults( this.build.in, { start: 0, end: 1000 }, ANIMATION_DEFAULTS );
			_.defaults( this.build.out, { start: 5000, end: 5000 }, ANIMATION_DEFAULTS );

			this.build.in.effect = 'jsn-es-anim-' + (this.build.in.fade ? 'fade-' : '') + this.build.in.effect;
			this.build.out.effect = 'jsn-es-anim-' + (this.build.out.fade ? 'fade-' : '') + this.build.out.effect;

			if ( this.type == 'video' ) {
				this.provider = this.$el.data( 'provider' );
				this.autoplay = this.$el.data( 'autoplay' );
				this.controls = this.$el.data( 'controls' );
				this.quality = this.$el.data( 'quality' ) || 'default';
				this.volume = this.$el.data( 'volume' ) || 100 ;
				if (!this.provider) {
					this.video = this.$('video').get(0);
					this.video.volume = this.volume/100;
				}
			}

			this.$link = this.$el.children('a').remove();

			var $html = this.$el.html();
			this.$content = (this.$link.length ? this.$link : $( '<div>' ))
				.html( this.type == 'text' ? '<div>'+$html+'</div>' : $html );
			this.$animation = $( '<div class="item-animation">' ).append( this.$content );
			this.$el.empty().append( this.$animation );

			this.$el.addClass( 'hidden' );

			$( window ).on( 'ready:youtube:api', _.bind( this.initYoutubePlayer, this ) );
		},
		getStyle: function () {
			switch ( this.slide.responsive ) {
				case 'default':
					return this.style;
				case 'tablet':
					return _( {} ).extend( this.style, this.styleT );
				case 'mobile':
					return _( {} ).extend( this.style, this.styleM );
			}
		},

		resetCSS: function () {
			this.$el.attr( 'class', this.className ).get( 0 ).style.cssText = null;
			this.$content.attr( 'class', 'item-content ' + this.customClass ).get( 0 ).style.cssText = null;
			this.$animation.attr( 'class', 'item-animation' ).get( 0 ).style.cssText = null;
			return this.render();
		},
		render: function () {

			log( 'Item :: render');
			var style = this.getStyle();

			this.$el
				.addClass( 'jsn-es-item-' + this.type )
				.removeAttr( 'style' )
				.css( _( style ).pick( 'top', 'left', 'width', 'height', 'zIndex' ) )
				.css( {
					//transform: 'translate('+(this.origin[ 0 ]*-100)+'%,'+(this.origin[ 1 ]*-100)+'%)',
					marginLeft: -this.$el.outerWidth() * this.origin[ 0 ] + 'px',
					marginTop: -this.$el.outerHeight() * this.origin[ 1 ] + 'px',
				} );
			this.$content
				.removeAttr( 'style' )
				.css( _( style ).omit( 'top', 'left', 'width', 'height', 'zIndex' ) );
		},

		start: function ( delay, duration ) {
			this.$el.addClass( 'hidden' );
			_( [ 'in', 'out' ] ).each( function ( type ) {
				var build = this.build[ type ];

				// When the build in time equals 0 then show all the items.
				if ( type == 'in' && build.start == build.end && build.start == 0 ) {
					this.$el.removeClass( 'hidden' );
					this.resetCSS();
				}
				// If the build out start == build out end == slide transition delay
				// then the item stay on the slide without hiding
				if ( type == 'in' || (type == 'out' && (build.start != build.end || build.end < duration)) ) {
					this.delay( function () {

						log( 'animating', type, build.start, build.end, duration );

						this.$el.removeClass( 'hidden' );
						this.resetCSS();

						type == 'in' ?
							this.$el.addClass( 'item-animating-in' ) :
							this.$el.addClass( 'item-animating-out' );

						this.$( '.item-animation' )
							.css('transform-origin', build.origin)
							.playKeyframe( {
								name: build.effect,
								duration: build.end - build.start + 'ms',
								timingFunction: build.timing,
								direction: type == 'in' ? 'normal' : 'reverse',
								complete: _.bind( function () {
									if ( type == 'in' && this.type == 'video' )
										this.autoplay && this.playVideo();
									if ( type == 'out' && this.type == 'video' )
										this.stopVideo();
								}, this )
							} );

					}, build.start + delay );
				}
			}, this );
			return this;
		},
		end: function () {
			if ( this.type == 'video' ) {
				this.stopVideo();
			}
			_( this.timer ).invoke( 'stop' );
			while ( this.timer.length )
				this.timer.pop();
			return this;
		},

		pause: function () {
			_( this.timer ).invoke( 'pause' );
		},
		resume: function () {
			_( this.timer ).invoke( 'start' );
		},

		initYoutubePlayer: function () {

			if ( this.type != 'video' || this.provider != 'youtube' || this.ytPlayerID )
				return;
			var IDRegex = /(?:youtu\.be\/|youtube\.com(?:\/embed\/|\/v\/|\/watch\?v=|\/user\/\S+|\/ytscreeningroom\?v=|\/sandalsResorts#\w\/\w\/.*\/))([^\/&]{10,12})/;

			this.ytPlayerID = _.uniqueId( 'jsn-es-youtube-player-' );
			this.ytVideoURL = this.$('a')
				.before($('<div>').attr( 'id', this.ytPlayerID ))
				.remove()
				.attr('href');
			this.ytVideoID = this.ytVideoURL.match(IDRegex);
			if (!this.ytVideoID)
				return;
			this.ytVideoID = this.ytVideoID[1];

			this.ytPlayer = new YT.Player( this.ytPlayerID, {
				videoId: this.ytVideoID,
				width: '100%',
				height: '100%',
				playerVars: {
					controls: 0,
					disablekb: 1,
					enablejsapi: 1,
					loop: 0,
					autoplay: 0,
					modestbranding: 1,
					cc_load_policy: 0,
					iv_load_policy: 3,
					showinfo: 0,
					rel: 1,
					fs: 0
				},
				events: {
					'onReady': _.bind( function () {

						this.ytPlayer.setVolume( this.volume );

						if ( this.slide.$el.is( '.active' ) && this.autoplay )
							this.playVideo();

					}, this ),
					'onStateChange': _.bind( function () {

						switch (this.ytPlayer.getPlayerState()) {
							case YT.PlayerState.PLAYING:
								console.log('Youtube video playing');
								this.slide.pause();
								break;
						}

					}, this )
				}
			} );

			log( 'Init new youtube player ', this.ytPlayerID, this.ytPlayer );
		},
		seekVideo: function ( time ) {
			this.slide.pause();
			switch ( this.provider ) {
				case 'youtube':
					this.ytPlayer &&
					this.ytPlayer.seekTo &&
					this.ytPlayer.seekTo( time );
					break;
				default :
					this.video.currentTime = 0;
					break;
			}
		},
		playVideo: function () {
			log('play video');
			this.seekVideo( 0 );
			switch ( this.provider ) {
				case 'youtube':
					this.ytPlayer &&
					this.ytPlayer.playVideo &&
					this.ytPlayer.playVideo();
					break;
				default:
					this.video.play();
					break;
			}
		},
		stopVideo: function () {
			switch ( this.provider ) {
				case 'youtube':
					this.ytPlayer &&
					this.ytPlayer.stopVideo &&
					this.ytPlayer.stopVideo();
					break;
				default :
					this.video.pause();
					break;
			}
		},

		delay: function ( cb, delay, context ) {
			var timer = new Timer( {
				onend: _.bind( cb, context || this )
			} );
			this.timer.push( timer.start( delay / 1000 ) );
			return timer;
		},
	} );

	var JSNEasySlider_TransitionView = Backbone.View.extend( {

		defaultOptions: TRANSITION_DEFAULTS,
		className: 'jsn-es-ts-view',
		initialize: function () {
			this.$el.addClass( 'hidden' );
		},
		play: function ( fromElement, toElement, options, callback ) {

			this.$el.empty().removeClass( 'hidden' );

			var options = _( {} ).defaults( options, this.defaultOptions );

			var $el1 = $( fromElement ).removeClass( 'hidden' );
			var $el2 = $( toElement ).removeClass( 'hidden' );

			var type = parseInt( options.type );

			var rows = options.rows;
			var cols = options.cols;

			var duration = options.duration;

			if ( type == 1 ) {
				$el1
					.css( TRANSITIONS[ options.effect + '-in' ][ '0%' ] )
					.playKeyframe( {
						name: [ 'jsn-es-transition-', options.effect, '-out' ].join( '' ),
						duration: duration + 'ms',
						timingFunction: 'ease'
					} );

				$el2
					.css( TRANSITIONS[ options.effect + '-out' ][ '0%' ] )
					.playKeyframe( {
						name: [ 'jsn-es-transition-', options.effect, '-in' ].join( '' ),
						duration: duration + 'ms',
						timingFunction: 'ease'
					} );
				return;
			}

			var tilesCount = rows * cols;

			var delayY = options.delayY;
			var delayX = options.delayX;
			var delayRandom = options.delayRandom;

			var width = this.$el.width();
			var height = this.$el.height();

			var tileWidth = (width / cols);
			var tileHeight = (height / rows);
			var tileWidthPercent = (100 / cols);
			var tileHeightPercent = (100 / rows);
			var tileDepth = options.cubeDepth;

			var alternateRotation;
			if ( options.cubeRotate == 'alternate' ) {
				options.cubeRotate = 1;
				alternateRotation = true;
			}

			var $wrapper = $( '<div>' )
				.addClass( 'jsn-es-ts-wrapper jsn-es-3d' );

			//var maxDelay = (Math.abs(delayX * cols) + Math.abs(delayY * rows));
			//duration -= (maxDelay);

			var animationComplete = _.after( tilesCount, function () {
				typeof callback != 'function' || callback();
			} );

			_( rows ).times( function ( row ) {
				_( cols ).times( function ( col ) {

					// Get tile position based on #col and #row
					var left = tileWidth * col;
					var top = tileHeight * row;
					var leftPercent = tileWidthPercent * col;
					var topPercent = tileHeightPercent * row;

					// Calculate actual delay time for this tile
					if ( !delayRandom ) {
						var tileDelayX = delayX >= 0 ? (delayX * col) :
						(Math.abs( delayX ) * cols) - (Math.abs( delayX ) * col);
						var tileDelayY = delayY >= 0 ? (delayY * row) :
						(Math.abs( delayY ) * rows) - (Math.abs( delayY ) * row);
						var delay = tileDelayX + tileDelayY;
					}
					else {
						var delay = _.random( 0, delayX + delayY );
					}

					var $tile = $( '<div>' ).appendTo( $wrapper )
						.addClass( 'jsn-es-tile' )
						.css( {
							left: leftPercent + '%',
							top: topPercent + '%',
							width: Math.ceil( tileWidth ) + 'px',
							height: Math.ceil( tileHeight ) + 'px',
						} );

					var $c1 = $el1.clone()
						.css( {
							position: 'absolute',
							width: width + 'px',
							left: -left + 'px',
							top: -top + 'px'
						} );

					$c1.find( '.jsn-es-item-video iframe' ).remove();

					var $c2 = $el2.clone()
						.css( {
							position: 'absolute',
							width: width + 'px',
							left: -left + 'px',
							top: -top + 'px'
						} );

					$c2.find( '.jsn-es-item-video iframe' ).remove();

					if ( type == 2 || type == 1 ) {

						var $in = $( '<div class="jsn-es-tile-wrapper in">' )
							.appendTo( $tile )
							.append( $c2 )
							.css( TRANSITIONS[ options.effect + '-in' ][ '0%' ] );

						var $out = $( '<div class="jsn-es-tile-wrapper out">' )
							.appendTo( $tile )
							.append( $c1 )
							.css( TRANSITIONS[ options.effect + '-out' ][ '0%' ] );

						$tile.addClass( 'jsn-es-tile-2d' );

						$out.playKeyframe( {
							name: [ 'jsn-es-transition-', options.effect, '-out' ].join( '' ),
							duration: duration + 'ms',
							timingFunction: 'ease',
							delay: delay + 'ms'
						} );
						$in.playKeyframe( {
							name: [ 'jsn-es-transition-', options.effect, '-in' ].join( '' ),
							duration: duration + 'ms',
							timingFunction: 'ease',
							delay: delay + 'ms',
							complete: animationComplete
						} );
					}
					if ( type == 3 ) {
						var axis = options.cubeAxis;
						if ( options.cubeFace == 'left' || options.cubeFace == 'right' ) {
							axis = 'y';
							tileDepth = 'width';
						}
						if ( options.cubeFace == 'top' || options.cubeFace == 'bottom' ) {
							axis = 'x';
							tileDepth = 'height';
						}
						if ( tileDepth == 'auto' && axis == 'x' ) {
							tileDepth = 'height';
						}

						switch ( tileDepth ) {
							case 'auto':
							case 'width':
								tileDepth = tileWidth;
								break;
							case 'height':
								tileDepth = tileHeight;
								break;
						}

						$tile.JSNES_Cuboid( tileWidth, tileHeight, tileDepth );

						var $faceFrom = $tile.find( '.jsn-es-cuboid-front' );
						var $faceTo = $tile.find( '.jsn-es-cuboid-' + options.cubeFace );

						$c1.appendTo( $faceFrom );
						$c2.appendTo( $faceTo );

						var angle;
						var name = options.cubeAnimation;
						var rotation = options.cubeRotate == 'random' ?
							([ -1, 1 ][ _.random( 0, 1 ) ]) : options.cubeRotate;
						if ( alternateRotation )
							rotation = (options.cubeRotate *= -1);
						if ( options.cubeFace == 'back' && axis == 'x' ) {
							$faceTo.css( 'transform', $faceTo.css( 'transform' ) + ' scale(-1)' )
						}

						switch ( options.cubeFace ) {
							case 'left':
								angle = rotation == 1 ? 90 : 270;
								break;
							case 'right':
								angle = rotation == 1 ? 270 : 90;
								break;
							case 'top':
								angle = rotation == 1 ? 270 : 90;
								break;
							case 'bottom':
								angle = rotation == 1 ? 90 : 270;
								break;
							case 'back':
								angle = 180;
								break;
						}
						$tile.find( '.jsn-es-cuboid' ).playKeyframe( {
							name: name + '-' + axis + '-' + angle + (rotation == 1 ? '' : '-o'),
							duration: duration + 'ms',
							timingFunction: 'ease',
							delay: delay + 'ms',
							complete: animationComplete
						} );
					}
				} );
			} );

			$el1.addClass( 'hidden' );
			$el2.addClass( 'hidden' );

			// Finally empty DOM and re-insert
			$wrapper.appendTo( this.el );
		}
	} );

	$.fn.JSNEasySlider = function ( method ) {
		var args = _( arguments ).slice( 1 );
		return this.each( function () {
			var slider = $( this ).data( 'jsn-easyslider-data' );
			if ( !method || typeof method == 'object' ) {
				if ( !slider ) {
					slider = new JSNEasySlider( _.extend( {}, method, { el: this } ) );
				}
				$( this ).data( 'jsn-easyslider-data', slider );
			}
			else if ( slider && typeof method == 'string' ) {
				slider[ method ].apply( slider, args );
			}
		} );
	};

	$.fn.JSNES_Cuboid = function ( width, height, depth ) {
		return this.each( function () {

			depth || (depth = width);

			var halfWidth = width / 2;
			var halfHeight = height / 2;
			var halfDepth = depth / 2;
			var setback = 0;

			var $cuboid = $( '<div class="jsn-es-cuboid">' ).appendTo( this )
				.css( {
					width: width + 'px',
					height: height + 'px'
				} );
			$( this ).css( {
				transform: 'translateZ(' + (-halfDepth) + 'px)'
			} )

			$( '<div class="jsn-es-cuboid-face jsn-es-cuboid-front">' ).appendTo( $cuboid )
				.css( {
					transform: 'translateZ(' + setback + 'px) translateZ(' + (halfDepth) + 'px)',
					width: width + 'px',
					height: height + 'px'
				} );
			$( '<div class="jsn-es-cuboid-face jsn-es-cuboid-back">' ).appendTo( $cuboid )
				.css( {
					transform: 'translateZ(' + setback + 'px) rotateY(180deg) translateZ(' + (halfDepth) + 'px)',
					width: width + 'px',
					height: height + 'px'
				} );
			$( '<div class="jsn-es-cuboid-face jsn-es-cuboid-left">' ).appendTo( $cuboid )
				.css( {
					transform: 'translateZ(' + setback + 'px) rotateY(-90deg) translateZ(' + (halfWidth) + 'px)',
					marginLeft: -halfDepth + 'px',
					width: depth + 'px',
					height: height + 'px'
				} );
			$( '<div class="jsn-es-cuboid-face jsn-es-cuboid-right">' ).appendTo( $cuboid )
				.css( {
					transform: 'translateZ(' + setback + 'px) rotateY(90deg) translateZ(' + (halfWidth) + 'px)',
					marginLeft: -halfDepth + 'px',
					width: depth + 'px',
					height: height + 'px'
				} );
			$( '<div class="jsn-es-cuboid-face jsn-es-cuboid-top">' ).appendTo( $cuboid )
				.css( {
					transform: 'translateZ(' + setback + 'px) rotateX(90deg) translateZ(' + halfHeight + 'px)',
					marginTop: -halfDepth + 'px',
					width: width + 'px',
					height: depth + 'px'
				} );
			$( '<div class="jsn-es-cuboid-face jsn-es-cuboid-bottom">' ).appendTo( $cuboid )
				.css( {
					transform: 'translateZ(' + setback + 'px) rotateX(-90deg) translateZ(' + halfHeight + 'px)',
					marginTop: -halfDepth + 'px',
					width: width + 'px',
					height: depth + 'px'
				} );
		} );
	};

	!function checkYoutubeReady() {
		if (typeof YT == 'undefined')
			return setTimeout(checkYoutubeReady, 1000);
		log( 'YOUTUBE API READY' )
		youtubeReady = true;
		$( window ).trigger( 'ready:youtube:api' );
	}();

	$.getScript( 'https://www.youtube.com/player_api' );

}( this, jQuery, _, Backbone );